#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""This is the dnf plugin for the kamp-simon-shell-client.

It is NOT intended to be run directly!
"""


import os
import dnf
from dnfpluginscore import _, logger


# Every plugin must be a subclass of dnf.Plugin.
class KampSimonShellClientPlugin(dnf.Plugin):
    # Every plugin must provide its name.
    name = 'kamp-simon-shell-client'

    # Every plugin must provide its own initialization function.
    def __init__(self, base, cli):
        """Initialize the plugin."""
        super().__init__(base, cli)
        self.base = base
        self.logger = logger

    # This hook is called immediately after the CLI/extension is finished configuring DNF.
    # The plugin can use this to tweak the global configuration or the repository configuration.
    def config(self):
        conf = self.read_config(self.base.conf)
        if conf.has_section('main'):
            if conf.has_option('main', 'package_change_flag_file'):
                self.package_change_flag_file = conf.get('main', 'package_change_flag_file')

    # This hook is called immediately after a successful transaction.
    def transaction(self):
        # Get flag file path from config.

        # Check if flag file was set.
        if not self.package_change_flag_file or self.package_change_flag_file is None:
            self.logger.error('No flag file given!')
            return

        # Get flag directory from flag file.
        flag_dir = os.path.dirname(self.package_change_flag_file)

        # Check if directory exists.
        if not os.path.isdir(flag_dir):
            try:
                # Create the directory and its parents.
                os.makedirs(flag_dir, 0777)
            except OSError as e:
                self.logger.error('Error while creating directory {dirname}: {err}'.fomat(dirname=flag_dir, err=str(e)))
                return

        # Check if the file exists.
        if not os.path.isfile(self.package_change_flag_file):
            try:
                # Create the file by opening and closing it.
                with open(self.package_change_flag_file, 'a') as _:
                    pass
            except OSError as e:
                self.logger.error('Error while creating file {filename}: {err}'.fomat(filename=self.package_change_flag_file, err=str(e)))
                return

        try:
            # Set access and modified times to the current time.
            os.utime(self.package_change_flag_file, None)
        except OSError as e:
            self.logger.error('Error while setting access and modified times of {filename}: {err}'.fomat(filename=self.package_change_flag_file, err=str(e)))
            return

        try:
            # Set directory permissions.
            os.chmod(flag_dir, 0777)
        except OSError as e:
            self.logger.error('Error while setting directory permissions of {dirname}: {err}'.fomat(dirname=flag_dir, err=str(e)))
            return

        try:
            # Set file permissions.
            os.chmod(self.package_change_flag_file, 0666)
        except OSError as e:
            self.logger.error('Error while setting file permissions of {filename}: {err}'.fomat(filename=self.package_change_flag_file, err=str(e)))
            return
